@@ -20,13 +20,16 @@ package ap
 import (
 	"net/url"
 
-	"github.com/superseriousbusiness/activity/streams/vocab"
+	"code.superseriousbusiness.org/activity/streams/vocab"
 )
 
 // IsActivityable returns whether AS vocab type name is acceptable as Activityable.
 func IsActivityable(typeName string) bool {
-	return isActivity(typeName) ||
-		isIntransitiveActivity(typeName)
+	return isActivity(typeName)
+	// See interfaces_test.go comment
+	// about intransitive activities:
+	//
+	// || isIntransitiveActivity(typeName)
 }
 
 // ToActivityable safely tries to cast vocab.Type as Activityable, also checking for expected AS type names.
@@ -73,7 +76,8 @@ func IsStatusable(typeName string) bool {
 		ObjectEvent,
 		ObjectPlace,
 		ObjectProfile,
-		ActivityQuestion:
+		ActivityQuestion,
+		ObjectAlbum:
 		return true
 	default:
 		return false
@@ -124,6 +128,96 @@ func ToPollOptionable(t vocab.Type) (PollOptionable, bool) {
 	return note, true
 }
 
+// IsAccept returns whether AS vocab type name
+// is something that can be cast to Acceptable.
+func IsAcceptable(typeName string) bool {
+	return typeName == ActivityAccept
+}
+
+// ToAcceptable safely tries to cast vocab.Type as Acceptable.
+func ToAcceptable(t vocab.Type) (Acceptable, bool) {
+	acceptable, ok := t.(vocab.ActivityStreamsAccept)
+	if !ok || !IsAcceptable(t.GetTypeName()) {
+		return nil, false
+	}
+	return acceptable, true
+}
+
+// IsAuthorizationable returns whether AS vocab type name
+// is something that can be cast to Authorizationable.
+func IsAuthorizationable(typeName string) bool {
+	switch typeName {
+	case ObjectLikeAuthorization,
+		ObjectReplyAuthorization,
+		ObjectAnnounceAuthorization:
+		return true
+	default:
+		return false
+	}
+}
+
+// ToAcceptable safely tries to cast vocab.Type as Approvable.
+func ToAuthorizationable(t vocab.Type) (Authorizationable, bool) {
+	authable, ok := t.(Authorizationable)
+	if !ok || !IsAuthorizationable(t.GetTypeName()) {
+		return nil, false
+	}
+	return authable, true
+}
+
+// IsAttachmentable returns whether AS vocab type name
+// is something that can be cast to Attachmentable.
+func IsAttachmentable(typeName string) bool {
+	switch typeName {
+	case ObjectAudio,
+		ObjectDocument,
+		ObjectImage,
+		ObjectVideo:
+		return true
+	default:
+		return false
+	}
+}
+
+// ToAttachmentable safely tries to cast vocab.Type as Attachmentable.
+func ToAttachmentable(t vocab.Type) (Attachmentable, bool) {
+	attachmentable, ok := t.(Attachmentable)
+	if !ok || !IsAttachmentable(t.GetTypeName()) {
+		return nil, false
+	}
+	return attachmentable, true
+}
+
+// IsAnnounceable returns whether AS vocab type name
+// is something that can be cast to vocab.ActivityStreamsAnnounce.
+func IsAnnounceable(typeName string) bool {
+	return typeName == ActivityAnnounce
+}
+
+// ToAnnounceable safely tries to cast vocab.Type as vocab.ActivityStreamsAnnounce.
+func ToAnnounceable(t vocab.Type) (vocab.ActivityStreamsAnnounce, bool) {
+	announceable, ok := t.(vocab.ActivityStreamsAnnounce)
+	if !ok || t.GetTypeName() != ActivityAnnounce {
+		return nil, false
+	}
+	return announceable, true
+}
+
+// IsLikeable returns whether AS vocab type name
+// is something that can be cast to vocab.ActivityStreamsLike.
+func IsLikeable(typeName string) bool {
+	return typeName == ActivityLike
+}
+
+// ToAnnouncToLikeableeable safely tries to cast vocab.Type as vocab.ActivityStreamsLike.
+func ToLikeable(t vocab.Type) (vocab.ActivityStreamsLike, bool) {
+	likeable, ok := t.(vocab.ActivityStreamsLike)
+	if !ok || t.GetTypeName() != ActivityLike {
+		return nil, false
+	}
+	return likeable, true
+}
+
 // Activityable represents the minimum activitypub interface for representing an 'activity'.
 // (see: IsActivityable() for types implementing this, though you MUST make sure to check
 // the typeName as this bare interface may be implementable by non-Activityable types).
@@ -163,9 +257,13 @@ type Accountable interface {
 	WithMovedTo
 	WithAlsoKnownAs
 	WithManuallyApprovesFollowers
+	WithHidesToPublicFromUnauthedWeb
+	WithHidesCcPublicFromUnauthedWeb
 	WithEndpoints
 	WithTag
 	WithPublished
+	WithUpdated
+	WithImage
 }
 
 // Statusable represents the minimum activitypub interface for representing a 'status'.
@@ -178,12 +276,12 @@ type Statusable interface {
 	WithName
 	WithInReplyTo
 	WithPublished
+	WithUpdated
 	WithURL
 	WithAttributedTo
 	WithTo
 	WithCc
 	WithSensitive
-	WithConversation
 	WithContent
 	WithAttachment
 	WithTag
@@ -217,6 +315,25 @@ type PollOptionable interface {
 	WithAttributedTo
 }
 
+// Acceptable represents the minimum activitypub
+// interface for representing an Accept.
+type Acceptable interface {
+	Activityable
+
+	WithTarget
+	WithResult
+}
+
+// Authorizationable represents the minimum interface for a
+// LikeAuthorization, ReplyAuthorization, AnnounceAuthorization.
+type Authorizationable interface {
+	vocab.Type
+
+	WithAttributedTo
+	WithInteractingObject
+	WithInteractionTarget
+}
+
 // Attachmentable represents the minimum activitypub interface for representing a 'mediaAttachment'. (see: IsAttachmentable).
 // This interface is fulfilled by: Audio, Document, Image, Video
 type Attachmentable interface {
@@ -300,6 +417,16 @@ type ReplyToable interface {
 	WithInReplyTo
 }
 
+// InteractionRequestable represents the minimum interface for an interaction request
+// activity, eg., LikeRequest, ReplyRequest, AnnounceRequest, QuoteRequest, etc..
+type InteractionRequestable interface {
+	vocab.Type
+
+	WithActor
+	WithObject
+	WithInstrument
+}
+
 // CollectionIterator represents the minimum interface for interacting with a
 // wrapped Collection or OrderedCollection in order to access next / prev items.
 type CollectionIterator interface {
@@ -307,6 +434,12 @@ type CollectionIterator interface {
 
 	NextItem() TypeOrIRI
 	PrevItem() TypeOrIRI
+
+	// TotalItems returns the total items
+	// present in the collection, derived
+	// from the totalItems property, or -1
+	// if totalItems not present / readable.
+	TotalItems() int
 }
 
 // CollectionPageIterator represents the minimum interface for interacting with a wrapped
@@ -319,6 +452,12 @@ type CollectionPageIterator interface {
 
 	NextItem() TypeOrIRI
 	PrevItem() TypeOrIRI
+
+	// TotalItems returns the total items
+	// present in the collection, derived
+	// from the totalItems property, or -1
+	// if totalItems not present / readable.
+	TotalItems() int
 }
 
 // Flaggable represents the minimum interface for an activitystreams 'Flag' activity.
@@ -396,6 +535,7 @@ type WithValue interface {
 // WithImage represents an activity with ActivityStreamsImageProperty
 type WithImage interface {
 	GetActivityStreamsImage() vocab.ActivityStreamsImageProperty
+	SetActivityStreamsImage(vocab.ActivityStreamsImageProperty)
 }
 
 // WithSummary represents an activity with ActivityStreamsSummaryProperty
@@ -512,10 +652,6 @@ type WithSensitive interface {
 	SetActivityStreamsSensitive(vocab.ActivityStreamsSensitiveProperty)
 }
 
-// WithConversation ...
-type WithConversation interface { // TODO
-}
-
 // WithContent represents an activity with ActivityStreamsContentProperty
 type WithContent interface {
 	GetActivityStreamsContent() vocab.ActivityStreamsContentProperty
@@ -552,9 +688,11 @@ type WithBlurhash interface {
 	SetTootBlurhash(vocab.TootBlurhashProperty)
 }
 
-// type withFocalPoint interface {
-// 	// TODO
-// }
+// WithFocalPoint represents an object with TootFocalPointProperty.
+type WithFocalPoint interface {
+	GetTootFocalPoint() vocab.TootFocalPointProperty
+	SetTootFocalPoint(vocab.TootFocalPointProperty)
+}
 
 // WithHref represents an activity with ActivityStreamsHrefProperty
 type WithHref interface {
@@ -580,6 +718,12 @@ type WithObject interface {
 	SetActivityStreamsObject(vocab.ActivityStreamsObjectProperty)
 }
 
+// WithInstrument represents an activity with ActivityStreamsInstrumentProperty
+type WithInstrument interface {
+	GetActivityStreamsInstrument() vocab.ActivityStreamsInstrumentProperty
+	SetActivityStreamsInstrument(vocab.ActivityStreamsInstrumentProperty)
+}
+
 // WithTarget represents an activity with ActivityStreamsTargetProperty
 type WithTarget interface {
 	GetActivityStreamsTarget() vocab.ActivityStreamsTargetProperty
@@ -610,6 +754,18 @@ type WithManuallyApprovesFollowers interface {
 	SetActivityStreamsManuallyApprovesFollowers(vocab.ActivityStreamsManuallyApprovesFollowersProperty)
 }
 
+// WithHidesToPublicFromUnauthedWeb represents a Person or profile with the hidesToPublicFromUnauthedWeb property.
+type WithHidesToPublicFromUnauthedWeb interface {
+	GetGoToSocialHidesToPublicFromUnauthedWeb() vocab.GoToSocialHidesToPublicFromUnauthedWebProperty
+	SetGoToSocialHidesToPublicFromUnauthedWeb(vocab.GoToSocialHidesToPublicFromUnauthedWebProperty)
+}
+
+// WithHidesCcPublicFromUnauthedWeb represents a Person or profile with the hidesCcPublicFromUnauthedWeb property.
+type WithHidesCcPublicFromUnauthedWeb interface {
+	GetGoToSocialHidesCcPublicFromUnauthedWeb() vocab.GoToSocialHidesCcPublicFromUnauthedWebProperty
+	SetGoToSocialHidesCcPublicFromUnauthedWeb(vocab.GoToSocialHidesCcPublicFromUnauthedWebProperty)
+}
+
 // WithEndpoints represents a Person or profile with the endpoints property
 type WithEndpoints interface {
 	GetActivityStreamsEndpoints() vocab.ActivityStreamsEndpointsProperty
@@ -645,3 +801,59 @@ type WithVotersCount interface {
 	GetTootVotersCount() vocab.TootVotersCountProperty
 	SetTootVotersCount(vocab.TootVotersCountProperty)
 }
+
+// WithReplies represents an object with GoToSocialInteractionPolicy.
+type WithInteractionPolicy interface {
+	GetGoToSocialInteractionPolicy() vocab.GoToSocialInteractionPolicyProperty
+	SetGoToSocialInteractionPolicy(vocab.GoToSocialInteractionPolicyProperty)
+}
+
+// WithPolicyRules represents an activity with always and approvalRequired properties.
+type WithPolicyRules interface {
+	GetGoToSocialAutomaticApproval() vocab.GoToSocialAutomaticApprovalProperty
+	GetGoToSocialManualApproval() vocab.GoToSocialManualApprovalProperty
+	GetGoToSocialAlways() vocab.GoToSocialAlwaysProperty                     // Deprecated
+	GetGoToSocialApprovalRequired() vocab.GoToSocialApprovalRequiredProperty // Deprecated
+}
+
+// WithApprovedBy represents an object with the approvedBy property.
+type WithApprovedBy interface {
+	GetGoToSocialApprovedBy() vocab.GoToSocialApprovedByProperty
+	SetGoToSocialApprovedBy(vocab.GoToSocialApprovedByProperty)
+}
+
+// WithLikeAuthorization represents a Likeable with the likeAuthorization property.
+type WithLikeAuthorization interface {
+	GetGoToSocialLikeAuthorization() vocab.GoToSocialLikeAuthorizationProperty
+	SetGoToSocialLikeAuthorization(vocab.GoToSocialLikeAuthorizationProperty)
+}
+
+// WithReplyAuthorization represents a statusable with the replyAuthorization property.
+type WithReplyAuthorization interface {
+	GetGoToSocialReplyAuthorization() vocab.GoToSocialReplyAuthorizationProperty
+	SetGoToSocialReplyAuthorization(vocab.GoToSocialReplyAuthorizationProperty)
+}
+
+// WithAnnounceAuthorization represents an Announceable with the announceAuthorization property.
+type WithAnnounceAuthorization interface {
+	GetGoToSocialAnnounceAuthorization() vocab.GoToSocialAnnounceAuthorizationProperty
+	SetGoToSocialAnnounceAuthorization(vocab.GoToSocialAnnounceAuthorizationProperty)
+}
+
+// WithResult represents an activity or object with the result property.
+type WithResult interface {
+	GetActivityStreamsResult() vocab.ActivityStreamsResultProperty
+	SetActivityStreamsResult(vocab.ActivityStreamsResultProperty)
+}
+
+// WithInteractingObject represents an activity or object with the InteractingObject property.
+type WithInteractingObject interface {
+	GetGoToSocialInteractingObject() vocab.GoToSocialInteractingObjectProperty
+	SetGoToSocialInteractingObject(vocab.GoToSocialInteractingObjectProperty)
+}
+
+// WithInteractionTarget represents an activity or object with the InteractionTarget property.
+type WithInteractionTarget interface {
+	GetGoToSocialInteractionTarget() vocab.GoToSocialInteractionTargetProperty
+	SetGoToSocialInteractionTarget(vocab.GoToSocialInteractionTargetProperty)
+}
