@@ -31,4 +31,7 @@ type Tag struct {
 	// Currently just a stub, if provided will always be an empty array.
 	// example: []
 	History *[]any `json:"history,omitempty"`
+	// Following is true if the user is following this tag, false if they're not,
+	// and not present if there is no currently authenticated user.
+	Following *bool `json:"following,omitempty"`
 }
