@@ -21,15 +21,16 @@ import (
 	"context"
 	"errors"
 	"slices"
-	"time"
+	"strings"
 
-	"github.com/superseriousbusiness/gotosocial/internal/db"
-	"github.com/superseriousbusiness/gotosocial/internal/gtscontext"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
-	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
-	"github.com/superseriousbusiness/gotosocial/internal/state"
-	"github.com/superseriousbusiness/gotosocial/internal/util"
+	"code.superseriousbusiness.org/gotosocial/internal/db"
+	"code.superseriousbusiness.org/gotosocial/internal/gtscontext"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/gotosocial/internal/gtsmodel"
+	"code.superseriousbusiness.org/gotosocial/internal/id"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
+	"code.superseriousbusiness.org/gotosocial/internal/state"
+	"code.superseriousbusiness.org/gotosocial/internal/util/xslices"
 	"github.com/uptrace/bun"
 )
 
@@ -51,7 +52,7 @@ func (s *statusDB) GetStatusByID(ctx context.Context, id string) (*gtsmodel.Stat
 
 func (s *statusDB) GetStatusesByIDs(ctx context.Context, ids []string) ([]*gtsmodel.Status, error) {
 	// Load all input status IDs via cache loader callback.
-	statuses, err := s.state.Caches.GTS.Status.LoadIDs("ID",
+	statuses, err := s.state.Caches.DB.Status.LoadIDs("ID",
 		ids,
 		func(uncached []string) ([]*gtsmodel.Status, error) {
 			// Preallocate expected length of uncached statuses.
@@ -76,7 +77,7 @@ func (s *statusDB) GetStatusesByIDs(ctx context.Context, ids []string) ([]*gtsmo
 	// Reorder the statuses by their
 	// IDs to ensure in correct order.
 	getID := func(s *gtsmodel.Status) string { return s.ID }
-	util.OrderBy(statuses, ids, getID)
+	xslices.OrderBy(statuses, ids, getID)
 
 	if gtscontext.Barebones(ctx) {
 		// no need to fully populate.
@@ -151,7 +152,7 @@ func (s *statusDB) GetStatusBoost(ctx context.Context, boostOfID string, byAccou
 
 func (s *statusDB) getStatus(ctx context.Context, lookup string, dbQuery func(*gtsmodel.Status) error, keyParts ...any) (*gtsmodel.Status, error) {
 	// Fetch status from database cache with loader callback
-	status, err := s.state.Caches.GTS.Status.LoadOne(lookup, func() (*gtsmodel.Status, error) {
+	status, err := s.state.Caches.DB.Status.LoadOne(lookup, func() (*gtsmodel.Status, error) {
 		var status gtsmodel.Status
 
 		// Not cached! Perform database query.
@@ -181,7 +182,7 @@ func (s *statusDB) getStatus(ctx context.Context, lookup string, dbQuery func(*g
 func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status) error {
 	var (
 		err  error
-		errs = gtserror.NewMultiError(9)
+		errs gtserror.MultiError
 	)
 
 	if status.Account == nil {
@@ -202,7 +203,7 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 				gtscontext.SetBarebones(ctx),
 				status.InReplyToID,
 			)
-			if err != nil {
+			if err != nil && !errors.Is(err, db.ErrNoEntries) {
 				errs.Appendf("error populating status parent: %w", err)
 			}
 		}
@@ -257,7 +258,7 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 	if !status.AttachmentsPopulated() {
 		// Status attachments are out-of-date with IDs, repopulate.
 		status.Attachments, err = s.state.DB.GetAttachmentsByIDs(
-			ctx, // these are already barebones
+			gtscontext.SetBarebones(ctx),
 			status.AttachmentIDs,
 		)
 		if err != nil {
@@ -268,7 +269,7 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 	if !status.TagsPopulated() {
 		// Status tags are out-of-date with IDs, repopulate.
 		status.Tags, err = s.state.DB.GetTags(
-			ctx,
+			gtscontext.SetBarebones(ctx),
 			status.TagIDs,
 		)
 		if err != nil {
@@ -279,7 +280,7 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 	if !status.MentionsPopulated() {
 		// Status mentions are out-of-date with IDs, repopulate.
 		status.Mentions, err = s.state.DB.GetMentions(
-			ctx, // leave fully populated for now
+			ctx, // TODO: manually populate mentions for places expecting these populated
 			status.MentionIDs,
 		)
 		if err != nil {
@@ -290,7 +291,7 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 	if !status.EmojisPopulated() {
 		// Status emojis are out-of-date with IDs, repopulate.
 		status.Emojis, err = s.state.DB.GetEmojisByIDs(
-			ctx, // these are already barebones
+			gtscontext.SetBarebones(ctx),
 			status.EmojiIDs,
 		)
 		if err != nil {
@@ -300,11 +301,12 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 
 	if status.CreatedWithApplicationID != "" && status.CreatedWithApplication == nil {
 		// Populate the status' expected CreatedWithApplication (not always set).
+		// Don't error on ErrNoEntries, as the application may have been cleaned up.
 		status.CreatedWithApplication, err = s.state.DB.GetApplicationByID(
-			ctx, // these are already barebones
+			gtscontext.SetBarebones(ctx),
 			status.CreatedWithApplicationID,
 		)
-		if err != nil {
+		if err != nil && !errors.Is(err, db.ErrNoEntries) {
 			errs.Appendf("error populating status application: %w", err)
 		}
 	}
@@ -312,163 +314,327 @@ func (s *statusDB) PopulateStatus(ctx context.Context, status *gtsmodel.Status)
 	return errs.Combine()
 }
 
+func (s *statusDB) PopulateStatusEdits(ctx context.Context, status *gtsmodel.Status) error {
+	var err error
+
+	if !status.EditsPopulated() {
+		// Status edits are out-of-date with IDs, repopulate.
+		status.Edits, err = s.state.DB.GetStatusEditsByIDs(
+			gtscontext.SetBarebones(ctx),
+			status.EditIDs,
+		)
+		if err != nil {
+			return gtserror.Newf("error populating status edits: %w", err)
+		}
+	}
+
+	return nil
+}
+
 func (s *statusDB) PutStatus(ctx context.Context, status *gtsmodel.Status) error {
-	return s.state.Caches.GTS.Status.Store(status, func() error {
+	return s.state.Caches.DB.Status.Store(status, func() error {
 		// It is safe to run this database transaction within cache.Store
 		// as the cache does not attempt a mutex lock until AFTER hook.
 		//
 		return s.db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
-			// create links between this status and any emojis it uses
-			for _, i := range status.EmojiIDs {
+			if status.BoostOfID != "" {
+				var threadID string
+
+				// Boost wrappers always inherit thread
+				// of the origin status they're boosting.
+				if err := tx.
+					NewSelect().
+					Table("statuses").
+					Column("thread_id").
+					Where("? = ?", bun.Ident("id"), status.BoostOfID).
+					Scan(ctx, &threadID); err != nil {
+					return gtserror.Newf("error selecting boosted status: %w", err)
+				}
+
+				// Set the selected thread.
+				status.ThreadID = threadID
+
+				// They also require no further
+				// checks! Simply insert status here.
+				return insertStatus(ctx, tx, status)
+			}
+
+			// Gather a list of possible thread IDs
+			// of all the possible related statuses
+			// to this one. If one exists we can use
+			// the end result, and if too many exist
+			// we can fix the status threading.
+			var threadIDs []string
+
+			if status.InReplyToID != "" {
+				var threadID string
+
+				// A stored parent status exists,
+				// select its thread ID to ideally
+				// inherit this for status.
+				if err := tx.
+					NewSelect().
+					Table("statuses").
+					Column("thread_id").
+					Where("? = ?", bun.Ident("id"), status.InReplyToID).
+					Scan(ctx, &threadID); err != nil {
+					return gtserror.Newf("error selecting status parent: %w", err)
+				}
+
+				// Append possible ID to threads slice.
+				threadIDs = append(threadIDs, threadID)
+
+			} else if status.InReplyToURI != "" {
+				var ids []string
+
+				// A parent status exists but is not
+				// yet stored. See if any siblings for
+				// this shared parent exist with their
+				// own thread IDs.
+				if err := tx.
+					NewSelect().
+					Table("statuses").
+					Column("thread_id").
+					Where("? = ?", bun.Ident("in_reply_to_uri"), status.InReplyToURI).
+					Scan(ctx, &ids); err != nil && !errors.Is(err, db.ErrNoEntries) {
+					return gtserror.Newf("error selecting status siblings: %w", err)
+				}
+
+				// Append possible IDs to threads slice.
+				threadIDs = append(threadIDs, ids...)
+			}
+
+			if !*status.Local {
+				var ids []string
+
+				// For remote statuses specifically, check to
+				// see if any children are stored for this new
+				// stored parent with their own thread IDs.
+				if err := tx.
+					NewSelect().
+					Table("statuses").
+					Column("thread_id").
+					Where("? = ?", bun.Ident("in_reply_to_uri"), status.URI).
+					Scan(ctx, &ids); err != nil && !errors.Is(err, db.ErrNoEntries) {
+					return gtserror.Newf("error selecting status children: %w", err)
+				}
+
+				// Append possible IDs to threads slice.
+				threadIDs = append(threadIDs, ids...)
+			}
+
+			// Ensure only *unique* posssible thread IDs.
+			threadIDs = xslices.Deduplicate(threadIDs)
+			switch len(threadIDs) {
+
+			case 0:
+				// No related status with thread ID already exists,
+				// so create new thread ID from status creation time.
+				threadID := id.NewULIDFromTime(status.CreatedAt)
+
+				// Insert new thread.
 				if _, err := tx.
 					NewInsert().
-					Model(&gtsmodel.StatusToEmoji{
-						StatusID: status.ID,
-						EmojiID:  i,
-					}).
-					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("status_id"), bun.Ident("emoji_id")).
+					Model(&gtsmodel.Thread{ID: threadID}).
 					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
+					return gtserror.Newf("error inserting thread: %w", err)
+				}
+
+				// Update status thread ID.
+				status.ThreadID = threadID
+
+			case 1:
+				// Inherit single known thread.
+				status.ThreadID = threadIDs[0]
+
+			default:
+				var err error
+				log.Infof(ctx, "reconciling status threading for %s: [%s]", status.URI, strings.Join(threadIDs, ","))
+				status.ThreadID, err = s.fixStatusThreading(ctx, tx, threadIDs)
+				if err != nil {
+					return err
 				}
 			}
 
-			// create links between this status and any tags it uses
-			for _, i := range status.TagIDs {
-				if _, err := tx.
-					NewInsert().
-					Model(&gtsmodel.StatusToTag{
-						StatusID: status.ID,
-						TagID:    i,
-					}).
-					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("status_id"), bun.Ident("tag_id")).
-					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
-				}
-			}
-
-			// change the status ID of the media attachments to the new status
-			for _, a := range status.Attachments {
-				a.StatusID = status.ID
-				a.UpdatedAt = time.Now()
-				if _, err := tx.
-					NewUpdate().
-					Model(a).
-					Column("status_id", "updated_at").
-					Where("? = ?", bun.Ident("media_attachment.id"), a.ID).
-					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
-				}
-			}
-
-			// If the status is threaded, create
-			// link between thread and status.
-			if status.ThreadID != "" {
-				if _, err := tx.
-					NewInsert().
-					Model(&gtsmodel.ThreadToStatus{
-						ThreadID: status.ThreadID,
-						StatusID: status.ID,
-					}).
-					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("thread_id"), bun.Ident("status_id")).
-					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
-				}
-			}
-
-			// Finally, insert the status
-			_, err := tx.NewInsert().Model(status).Exec(ctx)
-			return err
+			// And after threading, insert status.
+			// This will error if ThreadID is unset.
+			return insertStatus(ctx, tx, status)
 		})
 	})
 }
 
-func (s *statusDB) UpdateStatus(ctx context.Context, status *gtsmodel.Status, columns ...string) error {
-	status.UpdatedAt = time.Now()
-	if len(columns) > 0 {
-		// If we're updating by column, ensure "updated_at" is included.
-		columns = append(columns, "updated_at")
+// fixStatusThreading can be called to reconcile statuses in the same thread but known to be using multiple given threads.
+func (s *statusDB) fixStatusThreading(ctx context.Context, tx bun.Tx, threadIDs []string) (string, error) {
+	if len(threadIDs) <= 1 {
+		panic("invalid call to fixStatusThreading()")
 	}
 
-	return s.state.Caches.GTS.Status.Store(status, func() error {
+	// Sort ascending, i.e.
+	// oldest thread ID first.
+	slices.Sort(threadIDs)
+
+	// Drop the oldest thread ID
+	// from slice, we'll keep this.
+	threadID := threadIDs[0]
+	threadIDs = threadIDs[1:]
+
+	// On updates, gather IDs of changed model
+	// IDs for later stage of cache invalidation,
+	// preallocating slices for worst-case scenarios.
+	statusIDs := make([]string, 0, 4*len(threadIDs))
+	muteIDs := make([]string, 0, 4*len(threadIDs))
+
+	// Update all statuses with
+	// thread IDs to use oldest.
+	if _, err := tx.
+		NewUpdate().
+		Table("statuses").
+		Where("? IN (?)", bun.Ident("thread_id"), bun.In(threadIDs)).
+		Set("? = ?", bun.Ident("thread_id"), threadID).
+		Returning("?", bun.Ident("id")).
+		Exec(ctx, &statusIDs); err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return "", gtserror.Newf("error updating statuses: %w", err)
+	}
+
+	// Update all thread mutes with
+	// thread IDs to use oldest.
+	if _, err := tx.
+		NewUpdate().
+		Table("thread_mutes").
+		Where("? IN (?)", bun.Ident("thread_id"), bun.In(threadIDs)).
+		Set("? = ?", bun.Ident("thread_id"), threadID).
+		Returning("?", bun.Ident("id")).
+		Exec(ctx, &muteIDs); err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return "", gtserror.Newf("error updating thread mutes: %w", err)
+	}
+
+	// Delete all now
+	// unused thread IDs.
+	if _, err := tx.
+		NewDelete().
+		Table("threads").
+		Where("? IN (?)", bun.Ident("id"), bun.In(threadIDs)).
+		Exec(ctx); err != nil {
+		return "", gtserror.Newf("error deleting threads: %w", err)
+	}
+
+	// Invalidate caches for changed statuses and mutes.
+	s.state.Caches.DB.Status.InvalidateIDs("ID", statusIDs)
+	s.state.Caches.DB.ThreadMute.InvalidateIDs("ID", muteIDs)
+
+	return threadID, nil
+}
+
+// insertStatus handles the base status insert logic, that is the status itself,
+// any intermediary table links, and updating media attachments to point to status.
+func insertStatus(ctx context.Context, tx bun.Tx, status *gtsmodel.Status) error {
+
+	// create links between this
+	// status and any emojis it uses
+	for _, id := range status.EmojiIDs {
+		if _, err := tx.
+			NewInsert().
+			Model(&gtsmodel.StatusToEmoji{
+				StatusID: status.ID,
+				EmojiID:  id,
+			}).
+			Exec(ctx); err != nil {
+			return gtserror.Newf("error inserting status_to_emoji: %w", err)
+		}
+	}
+
+	// create links between this
+	// status and any tags it uses
+	for _, id := range status.TagIDs {
+		if _, err := tx.
+			NewInsert().
+			Model(&gtsmodel.StatusToTag{
+				StatusID: status.ID,
+				TagID:    id,
+			}).
+			Exec(ctx); err != nil {
+			return gtserror.Newf("error inserting status_to_tag: %w", err)
+		}
+	}
+
+	// change the status ID of the media
+	// attachments to the current status
+	for _, a := range status.Attachments {
+		a.StatusID = status.ID
+		a.ScheduledStatusID = ""
+		if _, err := tx.
+			NewUpdate().
+			Model(a).
+			Column("status_id", "scheduled_status_id").
+			Where("? = ?", bun.Ident("media_attachment.id"), a.ID).
+			Exec(ctx); err != nil {
+			return gtserror.Newf("error updating media: %w", err)
+		}
+	}
+
+	// Finally, insert the status
+	if _, err := tx.NewInsert().
+		Model(status).
+		Exec(ctx); err != nil {
+		return gtserror.Newf("error inserting status: %w", err)
+	}
+
+	return nil
+}
+
+func (s *statusDB) UpdateStatus(ctx context.Context, status *gtsmodel.Status, columns ...string) error {
+	return s.state.Caches.DB.Status.Store(status, func() error {
 		// It is safe to run this database transaction within cache.Store
 		// as the cache does not attempt a mutex lock until AFTER hook.
 		//
 		return s.db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
-			// create links between this status and any emojis it uses
-			for _, i := range status.EmojiIDs {
+
+			// create links between this
+			// status and any emojis it uses
+			for _, id := range status.EmojiIDs {
 				if _, err := tx.
 					NewInsert().
 					Model(&gtsmodel.StatusToEmoji{
 						StatusID: status.ID,
-						EmojiID:  i,
+						EmojiID:  id,
 					}).
 					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("status_id"), bun.Ident("emoji_id")).
 					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
+					return err
 				}
 			}
 
-			// create links between this status and any tags it uses
-			for _, i := range status.TagIDs {
+			// create links between this
+			// status and any tags it uses
+			for _, id := range status.TagIDs {
 				if _, err := tx.
 					NewInsert().
 					Model(&gtsmodel.StatusToTag{
 						StatusID: status.ID,
-						TagID:    i,
+						TagID:    id,
 					}).
 					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("status_id"), bun.Ident("tag_id")).
 					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
+					return err
 				}
 			}
 
-			// change the status ID of the media attachments to the new status
+			// change the status ID of the media
+			// attachments to the current status.
 			for _, a := range status.Attachments {
 				a.StatusID = status.ID
-				a.UpdatedAt = time.Now()
 				if _, err := tx.
 					NewUpdate().
 					Model(a).
+					Column("status_id").
 					Where("? = ?", bun.Ident("media_attachment.id"), a.ID).
 					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
-				}
-			}
-
-			// If the status is threaded, create
-			// link between thread and status.
-			if status.ThreadID != "" {
-				if _, err := tx.
-					NewInsert().
-					Model(&gtsmodel.ThreadToStatus{
-						ThreadID: status.ThreadID,
-						StatusID: status.ID,
-					}).
-					On("CONFLICT (?, ?) DO NOTHING", bun.Ident("thread_id"), bun.Ident("status_id")).
-					Exec(ctx); err != nil {
-					if !errors.Is(err, db.ErrAlreadyExists) {
-						return err
-					}
+					return err
 				}
 			}
 
 			// Finally, update the status
-			_, err := tx.
-				NewUpdate().
+			_, err := tx.NewUpdate().
 				Model(status).
 				Column(columns...).
 				Where("? = ?", bun.Ident("status.id"), status.ID).
@@ -479,25 +645,16 @@ func (s *statusDB) UpdateStatus(ctx context.Context, status *gtsmodel.Status, co
 }
 
 func (s *statusDB) DeleteStatusByID(ctx context.Context, id string) error {
-	// Load status into cache before attempting a delete,
-	// as we need it cached in order to trigger the invalidate
-	// callback. This in turn invalidates others.
-	_, err := s.GetStatusByID(
-		gtscontext.SetBarebones(ctx),
-		id,
-	)
-	if err != nil && !errors.Is(err, db.ErrNoEntries) {
-		// NOTE: even if db.ErrNoEntries is returned, we
-		// still run the below transaction to ensure related
-		// objects are appropriately deleted.
-		return err
-	}
+	// Gather necessary fields from
+	// deleted for cache invaliation.
+	var deleted gtsmodel.Status
+	deleted.ID = id
 
-	// On return ensure status invalidated from cache.
-	defer s.state.Caches.GTS.Status.Invalidate("ID", id)
+	// Delete status from database and any related links in a transaction.
+	if err := s.db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
 
-	return s.db.RunInTx(ctx, nil, func(ctx context.Context, tx bun.Tx) error {
-		// delete links between this status and any emojis it uses
+		// delete links between this
+		// status and any emojis it uses
 		if _, err := tx.
 			NewDelete().
 			TableExpr("? AS ?", bun.Ident("status_to_emojis"), bun.Ident("status_to_emoji")).
@@ -506,7 +663,8 @@ func (s *statusDB) DeleteStatusByID(ctx context.Context, id string) error {
 			return err
 		}
 
-		// delete links between this status and any tags it uses
+		// delete links between this
+		// status and any tags it uses
 		if _, err := tx.
 			NewDelete().
 			TableExpr("? AS ?", bun.Ident("status_to_tags"), bun.Ident("status_to_tag")).
@@ -515,28 +673,34 @@ func (s *statusDB) DeleteStatusByID(ctx context.Context, id string) error {
 			return err
 		}
 
-		// Delete links between this status
-		// and any threads it was a part of.
-		_, err = tx.
-			NewDelete().
-			TableExpr("? AS ?", bun.Ident("thread_to_statuses"), bun.Ident("thread_to_status")).
-			Where("? = ?", bun.Ident("thread_to_status.status_id"), id).
-			Exec(ctx)
-		if err != nil {
-			return err
-		}
-
 		// delete the status itself
 		if _, err := tx.
 			NewDelete().
-			TableExpr("? AS ?", bun.Ident("statuses"), bun.Ident("status")).
-			Where("? = ?", bun.Ident("status.id"), id).
-			Exec(ctx); err != nil {
+			Model(&deleted).
+			Where("? = ?", bun.Ident("id"), id).
+			Returning("?, ?, ?, ?, ?",
+				bun.Ident("account_id"),
+				bun.Ident("boost_of_id"),
+				bun.Ident("in_reply_to_id"),
+				bun.Ident("attachments"),
+				bun.Ident("poll_id"),
+			).
+			Exec(ctx); err != nil &&
+			!errors.Is(err, db.ErrNoEntries) {
 			return err
 		}
 
 		return nil
-	})
+	}); err != nil {
+		return err
+	}
+
+	// Invalidate cached status by its ID, manually
+	// call the invalidate hook in case not cached.
+	s.state.Caches.DB.Status.Invalidate("ID", id)
+	s.state.Caches.OnInvalidateStatus(&deleted)
+
+	return nil
 }
 
 func (s *statusDB) GetStatusesUsingEmoji(ctx context.Context, emojiID string) ([]*gtsmodel.Status, error) {
@@ -561,10 +725,15 @@ func (s *statusDB) GetStatusParents(ctx context.Context, status *gtsmodel.Status
 
 	for id := status.InReplyToID; id != ""; {
 		parent, err := s.GetStatusByID(ctx, id)
-		if err != nil {
+		if err != nil && !errors.Is(err, db.ErrNoEntries) {
 			return nil, err
 		}
 
+		if parent == nil {
+			// Parent status not found (e.g. deleted)
+			break
+		}
+
 		// Append parent status to slice
 		parents = append(parents, parent)
 
@@ -616,7 +785,7 @@ func (s *statusDB) CountStatusReplies(ctx context.Context, statusID string) (int
 }
 
 func (s *statusDB) getStatusReplyIDs(ctx context.Context, statusID string) ([]string, error) {
-	return s.state.Caches.GTS.InReplyToIDs.Load(statusID, func() ([]string, error) {
+	return s.state.Caches.DB.InReplyToIDs.Load(statusID, func() ([]string, error) {
 		var statusIDs []string
 
 		// Status reply IDs not in cache, perform DB query!
@@ -660,7 +829,7 @@ func (s *statusDB) CountStatusBoosts(ctx context.Context, statusID string) (int,
 }
 
 func (s *statusDB) getStatusBoostIDs(ctx context.Context, statusID string) ([]string, error) {
-	return s.state.Caches.GTS.BoostOfIDs.Load(statusID, func() ([]string, error) {
+	return s.state.Caches.DB.BoostOfIDs.Load(statusID, func() ([]string, error) {
 		var statusIDs []string
 
 		// Status boost IDs not in cache, perform DB query!
@@ -678,11 +847,136 @@ func (s *statusDB) getStatusBoostIDs(ctx context.Context, statusID string) ([]st
 	})
 }
 
-func (s *statusDB) IsStatusBookmarkedBy(ctx context.Context, status *gtsmodel.Status, accountID string) (bool, error) {
-	q := s.db.
+func (s *statusDB) MaxDirectStatusID(ctx context.Context) (string, error) {
+	maxID := ""
+	if err := s.db.
 		NewSelect().
-		TableExpr("? AS ?", bun.Ident("status_bookmarks"), bun.Ident("status_bookmark")).
-		Where("? = ?", bun.Ident("status_bookmark.status_id"), status.ID).
-		Where("? = ?", bun.Ident("status_bookmark.account_id"), accountID)
-	return exists(ctx, q)
+		Model((*gtsmodel.Status)(nil)).
+		ColumnExpr("COALESCE(MAX(?), '')", bun.Ident("id")).
+		Where("? = ?", bun.Ident("visibility"), gtsmodel.VisibilityDirect).
+		Scan(ctx, &maxID); // nocollapse
+	err != nil {
+		return "", err
+	}
+	return maxID, nil
+}
+
+func (s *statusDB) GetDirectStatusIDsBatch(ctx context.Context, minID string, maxIDInclusive string, count int) ([]string, error) {
+	var statusIDs []string
+	if err := s.db.
+		NewSelect().
+		Model((*gtsmodel.Status)(nil)).
+		Column("id").
+		Where("? = ?", bun.Ident("visibility"), gtsmodel.VisibilityDirect).
+		Where("? > ?", bun.Ident("id"), minID).
+		Where("? <= ?", bun.Ident("id"), maxIDInclusive).
+		Order("id ASC").
+		Limit(count).
+		Scan(ctx, &statusIDs); // nocollapse
+	err != nil {
+		return nil, err
+	}
+	return statusIDs, nil
+}
+
+func (s *statusDB) GetStatusInteractions(
+	ctx context.Context,
+	statusID string,
+	localOnly bool,
+) ([]gtsmodel.Interaction, error) {
+	// Prepare to get interactions.
+	interactions := []gtsmodel.Interaction{}
+
+	// Gather faves.
+	faves, err := s.state.DB.GetStatusFaves(ctx, statusID)
+	if err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return nil, err
+	}
+
+	for _, fave := range faves {
+		// Get account at least.
+		if fave.Account == nil {
+			fave.Account, err = s.state.DB.GetAccountByID(ctx, fave.AccountID)
+			if err != nil {
+				log.Errorf(ctx, "error getting account for fave: %v", err)
+				continue
+			}
+		}
+
+		if localOnly && !fave.Account.IsLocal() {
+			// Skip not local.
+			continue
+		}
+
+		interactions = append(interactions, fave)
+	}
+
+	// Gather replies.
+	replies, err := s.state.DB.GetStatusReplies(ctx, statusID)
+	if err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return nil, err
+	}
+
+	for _, reply := range replies {
+		// Get account at least.
+		if reply.Account == nil {
+			reply.Account, err = s.state.DB.GetAccountByID(ctx, reply.AccountID)
+			if err != nil {
+				log.Errorf(ctx, "error getting account for reply: %v", err)
+				continue
+			}
+		}
+
+		if localOnly && !reply.Account.IsLocal() {
+			// Skip not local.
+			continue
+		}
+
+		interactions = append(interactions, reply)
+	}
+
+	// Gather boosts.
+	boosts, err := s.state.DB.GetStatusBoosts(ctx, statusID)
+	if err != nil && !errors.Is(err, db.ErrNoEntries) {
+		return nil, err
+	}
+
+	for _, boost := range boosts {
+		// Get account at least.
+		if boost.Account == nil {
+			boost.Account, err = s.state.DB.GetAccountByID(ctx, boost.AccountID)
+			if err != nil {
+				log.Errorf(ctx, "error getting account for boost: %v", err)
+				continue
+			}
+		}
+
+		if localOnly && !boost.Account.IsLocal() {
+			// Skip not local.
+			continue
+		}
+
+		interactions = append(interactions, boost)
+	}
+
+	if len(interactions) == 0 {
+		return nil, db.ErrNoEntries
+	}
+
+	return interactions, nil
+}
+
+func (s *statusDB) GetStatusByEditID(
+	ctx context.Context,
+	editID string,
+) (*gtsmodel.Status, error) {
+	edit, err := s.state.DB.GetStatusEditByID(
+		gtscontext.SetBarebones(ctx),
+		editID,
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	return s.GetStatusByID(ctx, edit.StatusID)
 }
