@@ -18,25 +18,41 @@
 package typeutils
 
 import (
+	"cmp"
 	"context"
 	"errors"
 	"net/url"
+	"time"
 
+	"code.superseriousbusiness.org/gotosocial/internal/ap"
+	"code.superseriousbusiness.org/gotosocial/internal/config"
+	"code.superseriousbusiness.org/gotosocial/internal/db"
+	"code.superseriousbusiness.org/gotosocial/internal/gtserror"
+	"code.superseriousbusiness.org/gotosocial/internal/gtsmodel"
+	"code.superseriousbusiness.org/gotosocial/internal/log"
+	"code.superseriousbusiness.org/gotosocial/internal/uris"
+	"code.superseriousbusiness.org/gotosocial/internal/util"
 	"github.com/miekg/dns"
-	"github.com/superseriousbusiness/gotosocial/internal/ap"
-	"github.com/superseriousbusiness/gotosocial/internal/config"
-	"github.com/superseriousbusiness/gotosocial/internal/db"
-	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
-	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
-	"github.com/superseriousbusiness/gotosocial/internal/log"
-	"github.com/superseriousbusiness/gotosocial/internal/uris"
-	"github.com/superseriousbusiness/gotosocial/internal/util"
 )
 
-// ASRepresentationToAccount converts a remote account/person/application representation into a gts model account.
+// ASRepresentationToAccount converts a remote account / person
+// / application representation into a gts model account.
 //
-// If accountDomain is provided then this value will be used as the account's Domain, else the AP ID host.
-func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable ap.Accountable, accountDomain string) (*gtsmodel.Account, error) {
+// If accountDomain is provided then this value will be
+// used as the account's Domain, else the AP ID host.
+//
+// If accountUsername is provided then this is used as
+// a fallback when no preferredUsername is provided. Else
+// a lack of username will result in error return.
+func (c *Converter) ASRepresentationToAccount(
+	ctx context.Context,
+	accountable ap.Accountable,
+	accountDomain string,
+	accountUsername string,
+) (
+	*gtsmodel.Account,
+	error,
+) {
 	var err error
 
 	// Extract URI from accountable
@@ -54,26 +70,24 @@ func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable a
 	acct.URI = uri
 
 	// Check whether account is a usable actor type.
-	switch acct.ActorType = accountable.GetTypeName(); acct.ActorType {
-
-	// people, groups, and organizations aren't bots
-	case ap.ActorPerson, ap.ActorGroup, ap.ActorOrganization:
-		acct.Bot = util.Ptr(false)
-
-	// apps and services are
-	case ap.ActorApplication, ap.ActorService:
-		acct.Bot = util.Ptr(true)
-
-	// we don't know what this is!
-	default:
-		err := gtserror.Newf("unusable actor type for %s", uri)
+	actorTypeName := accountable.GetTypeName()
+	acct.ActorType = gtsmodel.ParseAccountActorType(actorTypeName)
+	if acct.ActorType == gtsmodel.AccountActorTypeUnknown {
+		err := gtserror.Newf("unusable actor type %s for %s", actorTypeName, uri)
 		return nil, gtserror.SetMalformed(err)
 	}
 
-	// Extract preferredUsername, this is a *requirement*.
-	acct.Username, err = ap.ExtractPreferredUsername(accountable)
-	if err != nil {
-		err := gtserror.Newf("unusable username for %s", uri)
+	// Set account username.
+	acct.Username = cmp.Or(
+
+		// Prefer the AP model provided username.
+		ap.ExtractPreferredUsername(accountable),
+
+		// Fallback username.
+		accountUsername,
+	)
+	if acct.Username == "" {
+		err := gtserror.Newf("missing username for %s", uri)
 		return nil, gtserror.SetMalformed(err)
 	}
 
@@ -89,6 +103,13 @@ func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable a
 		acct.UpdatedAt = pub
 	}
 
+	// Extract updated time if possible, i.e. last edited.
+	if upd := ap.GetUpdated(accountable); !upd.IsZero() {
+		acct.UpdatedAt = upd
+	} else {
+		acct.UpdatedAt = acct.CreatedAt
+	}
+
 	// Extract a preferred name (display name), fallback to username.
 	if displayName := ap.ExtractName(accountable); displayName != "" {
 		acct.DisplayName = displayName
@@ -119,7 +140,7 @@ func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable a
 	}
 
 	// account emojis (used in bio, display name, fields)
-	acct.Emojis, err = ap.ExtractEmojis(accountable)
+	acct.Emojis, err = ap.ExtractEmojis(accountable, acct.Domain)
 	if err != nil {
 		log.Warnf(ctx, "error(s) extracting account emojis for %s: %v", uri, err)
 	}
@@ -131,9 +152,9 @@ func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable a
 	acct.Note = ap.ExtractSummary(accountable)
 
 	// Assume not memorial (todo)
-	acct.Memorial = util.Ptr(false)
+	acct.MemorializedAt = time.Time{}
 
-	// Extract 'manuallyApprovesFollowers' aka locked account (default = true).
+	// Extract 'manuallyApprovesFollowers' aka locked account (default = false).
 	manuallyApprovesFollowers := ap.GetManuallyApprovesFollowers(accountable)
 	acct.Locked = &manuallyApprovesFollowers
 
@@ -223,6 +244,10 @@ func (c *Converter) ASRepresentationToAccount(ctx context.Context, accountable a
 	acct.PublicKey = pkey
 	acct.PublicKeyURI = pkeyURL.String()
 
+	// Web visibility for statuses.
+	acct.HidesToPublicFromUnauthedWeb = util.Ptr(ap.GetHidesToPublicFromUnauthedWeb(accountable))
+	acct.HidesCcPublicFromUnauthedWeb = util.Ptr(ap.GetHidesCcPublicFromUnauthedWeb(accountable))
+
 	return &acct, nil
 }
 
@@ -247,10 +272,12 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 	// status.URL
 	//
 	// Web URL of this status (optional).
-	if statusURL, err := ap.ExtractURL(statusable); err == nil {
-		status.URL = statusURL.String()
+	if statusURL := ap.GetURL(statusable); len(statusURL) != 0 {
+		// Take the first hit.
+		status.URL = statusURL[0].String()
 	} else {
-		status.URL = status.URI // Fall back to the URI.
+		// Fall back to the URI.
+		status.URL = status.URI
 	}
 
 	// status.Content
@@ -295,7 +322,7 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 	// status.Emojis
 	//
 	// Custom emojis for later dereferencing.
-	if emojis, err := ap.ExtractEmojis(statusable); err != nil {
+	if emojis, err := ap.ExtractEmojis(statusable, uriObj.Host); err != nil {
 		log.Warnf(ctx, "error extracting emojis for %s: %v", uri, err)
 	} else {
 		status.Emojis = emojis
@@ -326,18 +353,29 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 	// zero-time will fall back to db defaults.
 	if pub := ap.GetPublished(statusable); !pub.IsZero() {
 		status.CreatedAt = pub
-		status.UpdatedAt = pub
 	} else {
 		log.Warnf(ctx, "unusable published property on %s", uri)
+		status.CreatedAt = time.Now()
+	}
+
+	// status.Edited
+	//
+	// Extract and validate update time for status. Defaults to none.
+	if upd := ap.GetUpdated(statusable); !upd.Before(status.CreatedAt) {
+		status.EditedAt = upd
+	} else if !upd.IsZero() {
+
+		// This is a malformed status that will likely break our systems.
+		err := gtserror.Newf("status %s 'updated' predates 'published'", uri)
+		return nil, gtserror.SetMalformed(err)
 	}
 
 	// status.AccountURI
 	// status.AccountID
 	// status.Account
 	//
-	// Account that created the status. Assume we have
-	// this in the db by the time this function is called,
-	// error if we don't.
+	// Account that created the status. Assume we have this
+	// in the db by the time this function is called, else error.
 	status.Account, err = c.getASAttributedToAccount(ctx,
 		status.URI,
 		statusable,
@@ -393,15 +431,29 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 		return nil, gtserror.SetMalformed(err)
 	}
 
-	// Advanced visibility toggles for this status.
-	//
-	// TODO: a lot of work to be done here -- a new type
-	// needs to be created for this in go-fed/activity.
-	// Until this is implemented, assume all true.
+	// Status was sent to us or dereffed by
+	// us so it must be federated and not local.
 	status.Federated = util.Ptr(true)
-	status.Boostable = util.Ptr(true)
-	status.Replyable = util.Ptr(true)
-	status.Likeable = util.Ptr(true)
+	status.Local = util.Ptr(false)
+
+	// Derive interaction policy for this status.
+	status.InteractionPolicy = ap.ExtractInteractionPolicy(
+		statusable,
+		status.Account,
+	)
+
+	// Set approvedByURI if present,
+	// for later dereferencing.
+	if wab, ok := statusable.(ap.WithApprovedBy); ok {
+		approvedByURI := ap.GetApprovedBy(wab)
+		if approvedByURI != nil {
+			status.ApprovedByURI = approvedByURI.String()
+		}
+	}
+
+	// Assume not pending approval; this may
+	// change when permissivity is checked.
+	status.PendingApproval = util.Ptr(false)
 
 	// status.Sensitive
 	sensitive := ap.ExtractSensitive(statusable)
@@ -466,7 +518,9 @@ func (c *Converter) ASFollowToFollow(ctx context.Context, followable ap.Followab
 
 	follow := &gtsmodel.Follow{
 		URI:             uri,
+		Account:         origin,
 		AccountID:       origin.ID,
+		TargetAccount:   target,
 		TargetAccountID: target.ID,
 	}
 
@@ -502,6 +556,10 @@ func (c *Converter) ASLikeToFave(ctx context.Context, likeable ap.Likeable) (*gt
 		StatusID:        target.ID,
 		Status:          target,
 		URI:             uri,
+
+		// Assume not pending approval; this may
+		// change when permissivity is checked.
+		PendingApproval: util.Ptr(false),
 	}, nil
 }
 
@@ -591,9 +649,9 @@ func (c *Converter) ASAnnounceToStatus(
 	// zero-time will fall back to db defaults.
 	if pub := ap.GetPublished(announceable); !pub.IsZero() {
 		boost.CreatedAt = pub
-		boost.UpdatedAt = pub
 	} else {
 		log.Warnf(ctx, "unusable published property on %s", uri)
+		boost.CreatedAt = time.Now()
 	}
 
 	// Extract and load the boost actor account,
@@ -627,6 +685,10 @@ func (c *Converter) ASAnnounceToStatus(
 	boost.MentionIDs = make([]string, 0)
 	boost.EmojiIDs = make([]string, 0)
 
+	// Assume not pending approval; this may
+	// change when permissivity is checked.
+	boost.PendingApproval = util.Ptr(false)
+
 	// Remaining fields on the boost will be
 	// taken from the target status; it's not
 	// our job to do all that dereferencing here.
@@ -832,7 +894,7 @@ func (c *Converter) getASObjectStatus(ctx context.Context, id string, with ap.Wi
 	// Check for status in database with provided object URI.
 	status, err := c.state.DB.GetStatusByURI(ctx, object[0].String())
 	if err != nil {
-		return nil, gtserror.Newf("error getting object account from database: %w", err)
+		return nil, gtserror.Newf("error getting object status from database: %w", err)
 	}
 
 	return status, nil
