@@ -4,14 +4,46 @@ import (
 	"sync"
 
 	"codeberg.org/gruf/go-byteutil"
+	"codeberg.org/gruf/go-mangler/v2"
 )
 
 // Key represents one key to
 // lookup (potentially) stored
 // entries in an Index.
 type Key struct {
-	raw []any
 	key string
+	raw []any
+}
+
+// MakeKey generates Key{} from given parts.
+func MakeKey(parts ...any) Key {
+	buf := new_buffer()
+	buf.B = mangler.AppendMulti(buf.B[:0], parts...)
+	key := string(buf.B)
+	free_buffer(buf)
+	return Key{
+		raw: parts,
+		key: key,
+	}
+}
+
+// MakeKeys generates []Key{} from given (multiple) parts.
+func MakeKeys(parts ...[]any) []Key {
+	keys := make([]Key, len(parts))
+	if len(keys) != len(parts) {
+		panic(assert("BCE"))
+	}
+	buf := new_buffer()
+	for x, parts := range parts {
+		buf.B = mangler.AppendMulti(buf.B[:0], parts...)
+		key := string(buf.B)
+		keys[x] = Key{
+			raw: parts,
+			key: key,
+		}
+	}
+	free_buffer(buf)
+	return keys
 }
 
 // Key returns the underlying cache key string.
@@ -22,7 +54,7 @@ func (k Key) Key() string {
 
 // Equal returns whether keys are equal.
 func (k Key) Equal(o Key) bool {
-	return k.key == o.key
+	return (k.key == o.key)
 }
 
 // Value returns the raw slice of
@@ -31,11 +63,6 @@ func (k Key) Values() []any {
 	return k.raw
 }
 
-// Zero indicates a zero value key.
-func (k Key) Zero() bool {
-	return k.raw == nil
-}
-
 var buf_pool sync.Pool
 
 // new_buffer returns a new initialized byte buffer.
