@@ -1,15 +1,17 @@
 package schema
 
 import (
+	"cmp"
 	"database/sql"
 	"fmt"
 	"reflect"
+	"slices"
 	"strings"
-	"sync"
 	"time"
 
 	"github.com/jinzhu/inflection"
 
+	"github.com/uptrace/bun/dialect/feature"
 	"github.com/uptrace/bun/internal"
 	"github.com/uptrace/bun/internal/tagparser"
 )
@@ -23,7 +25,7 @@ const (
 )
 
 var (
-	baseModelType      = reflect.TypeOf((*BaseModel)(nil)).Elem()
+	baseModelType      = reflect.TypeFor[BaseModel]()
 	tableNameInflector = inflection.Plural
 )
 
@@ -46,42 +48,53 @@ type Table struct {
 	TypeName  string
 	ModelName string
 
+	Schema            string
 	Name              string
 	SQLName           Safe
 	SQLNameForSelects Safe
 	Alias             string
 	SQLAlias          Safe
 
+	allFields  []*Field // all fields including scanonly
 	Fields     []*Field // PKs + DataFields
 	PKs        []*Field
 	DataFields []*Field
+	relFields  []*Field
 
-	fieldsMapMu sync.RWMutex
-	FieldMap    map[string]*Field
+	FieldMap  map[string]*Field
+	StructMap map[string]*structField
 
-	Relations map[string]*Relation
-	Unique    map[string][]*Field
+	IsM2MTable bool // If true, this table is the "junction table" of an m2m relation.
+	Relations  map[string]*Relation
+	Unique     map[string][]*Field
 
 	SoftDeleteField       *Field
 	UpdateSoftDeleteField func(fv reflect.Value, tm time.Time) error
 
-	allFields []*Field // read only
-
 	flags internal.Flag
 }
 
-func newTable(dialect Dialect, typ reflect.Type) *Table {
-	t := new(Table)
-	t.dialect = dialect
-	t.Type = typ
-	t.ZeroValue = reflect.New(t.Type).Elem()
-	t.ZeroIface = reflect.New(t.Type).Interface()
-	t.TypeName = internal.ToExported(t.Type.Name())
-	t.ModelName = internal.Underscore(t.Type.Name())
-	tableName := tableNameInflector(t.ModelName)
-	t.setName(tableName)
-	t.Alias = t.ModelName
-	t.SQLAlias = t.quoteIdent(t.ModelName)
+type structField struct {
+	Index []int
+	Table *Table
+}
+
+func (table *Table) init(dialect Dialect, typ reflect.Type) {
+	table.dialect = dialect
+	table.Type = typ
+	table.ZeroValue = reflect.New(table.Type).Elem()
+	table.ZeroIface = reflect.New(table.Type).Interface()
+	table.TypeName = internal.ToExported(table.Type.Name())
+	table.ModelName = internal.Underscore(table.Type.Name())
+	tableName := tableNameInflector(table.ModelName)
+	table.setName(tableName)
+	table.Alias = table.ModelName
+	table.SQLAlias = table.quoteIdent(table.ModelName)
+	table.Schema = dialect.DefaultSchema()
+
+	table.Fields = make([]*Field, 0, typ.NumField())
+	table.FieldMap = make(map[string]*Field, typ.NumField())
+	table.processFields(typ)
 
 	hooks := []struct {
 		typ  reflect.Type
@@ -89,45 +102,236 @@ func newTable(dialect Dialect, typ reflect.Type) *Table {
 	}{
 		{beforeAppendModelHookType, beforeAppendModelHookFlag},
 
-		{beforeScanHookType, beforeScanHookFlag},
-		{afterScanHookType, afterScanHookFlag},
-
 		{beforeScanRowHookType, beforeScanRowHookFlag},
 		{afterScanRowHookType, afterScanRowHookFlag},
 	}
 
-	typ = reflect.PtrTo(t.Type)
+	typ = reflect.PointerTo(table.Type)
 	for _, hook := range hooks {
 		if typ.Implements(hook.typ) {
-			t.flags = t.flags.Set(hook.flag)
+			table.flags = table.flags.Set(hook.flag)
+		}
+	}
+}
+
+func (t *Table) processFields(typ reflect.Type) {
+	type embeddedField struct {
+		prefix     string
+		index      []int
+		unexported bool
+		subtable   *Table
+		subfield   *Field
+	}
+
+	names := make(map[string]struct{})
+	embedded := make([]embeddedField, 0, 10)
+	ebdStructs := make(map[string]*structField, 0)
+
+	for i, n := 0, typ.NumField(); i < n; i++ {
+		sf := typ.Field(i)
+		unexported := sf.PkgPath != ""
+
+		tagstr := sf.Tag.Get("bun")
+		if tagstr == "-" {
+			names[sf.Name] = struct{}{}
+			continue
+		}
+		tag := tagparser.Parse(tagstr)
+
+		if unexported && !sf.Anonymous { // unexported
+			continue
+		}
+
+		if sf.Anonymous {
+			if sf.Name == "BaseModel" && sf.Type == baseModelType {
+				t.processBaseModelField(sf)
+				continue
+			}
+
+			sfType := sf.Type
+			if sfType.Kind() == reflect.Ptr {
+				sfType = sfType.Elem()
+			}
+
+			if sfType.Kind() != reflect.Struct { // ignore unexported non-struct types
+				continue
+			}
+
+			subtable := t.dialect.Tables().InProgress(sfType)
+
+			for _, subfield := range subtable.allFields {
+				embedded = append(embedded, embeddedField{
+					index:      sf.Index,
+					unexported: unexported,
+					subtable:   subtable,
+					subfield:   subfield,
+				})
+			}
+			if len(subtable.StructMap) > 0 {
+				for k, v := range subtable.StructMap {
+					// NOTE: conflict Struct name
+					if _, ok := ebdStructs[k]; !ok {
+						ebdStructs[k] = &structField{
+							Index: makeIndex(sf.Index, v.Index),
+							Table: v.Table,
+						}
+					}
+				}
+			}
+
+			if tagstr != "" {
+				tag := tagparser.Parse(tagstr)
+				if tag.HasOption("inherit") || tag.HasOption("extend") {
+					t.Name = subtable.Name
+					t.TypeName = subtable.TypeName
+					t.SQLName = subtable.SQLName
+					t.SQLNameForSelects = subtable.SQLNameForSelects
+					t.Alias = subtable.Alias
+					t.SQLAlias = subtable.SQLAlias
+					t.ModelName = subtable.ModelName
+				}
+			}
+
+			continue
+		}
+
+		if prefix, ok := tag.Option("embed"); ok {
+			fieldType := indirectType(sf.Type)
+			if fieldType.Kind() != reflect.Struct {
+				panic(fmt.Errorf("bun: embed %s.%s: got %s, wanted reflect.Struct",
+					t.TypeName, sf.Name, fieldType.Kind()))
+			}
+
+			subtable := t.dialect.Tables().InProgress(fieldType)
+			for _, subfield := range subtable.allFields {
+				embedded = append(embedded, embeddedField{
+					prefix:     prefix,
+					index:      sf.Index,
+					unexported: unexported,
+					subtable:   subtable,
+					subfield:   subfield,
+				})
+			}
+			if len(subtable.StructMap) > 0 {
+				for k, v := range subtable.StructMap {
+					// NOTE: conflict Struct name
+					k = prefix + k
+					if _, ok := ebdStructs[k]; !ok {
+						ebdStructs[k] = &structField{
+							Index: makeIndex(sf.Index, v.Index),
+							Table: subtable,
+						}
+					}
+				}
+			}
+			continue
+		}
+
+		field := t.newField(sf, tag)
+		t.addField(field)
+		names[field.Name] = struct{}{}
+
+		if field.IndirectType.Kind() == reflect.Struct {
+			if t.StructMap == nil {
+				t.StructMap = make(map[string]*structField)
+			}
+			t.StructMap[field.Name] = &structField{
+				Index: field.Index,
+				Table: t.dialect.Tables().InProgress(field.IndirectType),
+			}
 		}
 	}
 
-	// Deprecated.
-	deprecatedHooks := []struct {
-		typ  reflect.Type
-		flag internal.Flag
-		msg  string
-	}{
-		{beforeScanHookType, beforeScanHookFlag, "rename BeforeScan hook to BeforeScanRow"},
-		{afterScanHookType, afterScanHookFlag, "rename AfterScan hook to AfterScanRow"},
+	// Only unambiguous embedded fields must be serialized.
+	ambiguousNames := make(map[string]int)
+	ambiguousTags := make(map[string]int)
+
+	// Embedded types can never override a field that was already present at
+	// the top-level.
+	for name := range names {
+		ambiguousNames[name]++
+		ambiguousTags[name]++
 	}
-	for _, hook := range deprecatedHooks {
-		if typ.Implements(hook.typ) {
-			internal.Deprecated.Printf("%s: %s", t.TypeName, hook.msg)
-			t.flags = t.flags.Set(hook.flag)
+
+	for _, f := range embedded {
+		ambiguousNames[f.prefix+f.subfield.Name]++
+		if !f.subfield.Tag.IsZero() {
+			ambiguousTags[f.prefix+f.subfield.Name]++
 		}
 	}
 
-	return t
+	for _, embfield := range embedded {
+		if ambiguousNames[embfield.prefix+embfield.subfield.Name] > 1 &&
+			!(!embfield.subfield.Tag.IsZero() && ambiguousTags[embfield.prefix+embfield.subfield.Name] == 1) {
+			continue // ambiguous embedded field
+		}
+
+		subfield := embfield.subfield.Clone()
+
+		subfield.Index = makeIndex(embfield.index, subfield.Index)
+		if embfield.prefix != "" {
+			subfield.Name = embfield.prefix + subfield.Name
+			subfield.SQLName = t.quoteIdent(subfield.Name)
+		}
+		t.addField(subfield)
+		if v, ok := subfield.Tag.Options["unique"]; ok {
+			t.addUnique(subfield, embfield.prefix, v)
+		}
+	}
+
+	if len(ebdStructs) > 0 && t.StructMap == nil {
+		t.StructMap = make(map[string]*structField)
+	}
+	for name, sfield := range ebdStructs {
+		if _, ok := t.StructMap[name]; !ok {
+			t.StructMap[name] = sfield
+		}
+	}
+
+	if len(embedded) > 0 {
+		// https://github.com/uptrace/bun/issues/1095
+		// < v1.2, all fields follow the order corresponding to the struct
+		// >= v1.2, < v1.2.8, fields of nested structs have been moved to the end.
+		// >= v1.2.8, The default behavior remains the same as initially,
+		sortFieldsByStruct(t.allFields)
+		sortFieldsByStruct(t.Fields)
+		sortFieldsByStruct(t.PKs)
+		sortFieldsByStruct(t.DataFields)
+	}
 }
 
-func (t *Table) init1() {
-	t.initFields()
+func sortFieldsByStruct(fields []*Field) {
+	slices.SortFunc(fields, func(left, right *Field) int {
+		for k := 0; k < len(left.Index) && k < len(right.Index); k++ {
+			if res := cmp.Compare(left.Index[k], right.Index[k]); res != 0 {
+				return res
+			}
+		}
+		// NOTE: should not reach
+		return 0
+	})
 }
 
-func (t *Table) init2() {
-	t.initRelations()
+func (t *Table) addUnique(field *Field, prefix string, tagOptions []string) {
+	var names []string
+	if len(tagOptions) == 1 {
+		// Split the value by comma, this will allow multiple names to be specified.
+		// We can use this to create multiple named unique constraints where a single column
+		// might be included in multiple constraints.
+		names = strings.Split(tagOptions[0], ",")
+	} else {
+		names = tagOptions
+	}
+
+	for _, uname := range names {
+		if t.Unique == nil {
+			t.Unique = make(map[string][]*Field)
+		}
+		if uname != "" && prefix != "" {
+			uname = prefix + uname
+		}
+		t.Unique[uname] = append(t.Unique[uname], field)
+	}
 }
 
 func (t *Table) setName(name string) {
@@ -152,30 +356,67 @@ func (t *Table) CheckPKs() error {
 }
 
 func (t *Table) addField(field *Field) {
+	t.allFields = append(t.allFields, field)
+
+	if field.Tag.HasOption("rel") || field.Tag.HasOption("m2m") {
+		t.relFields = append(t.relFields, field)
+		return
+	}
+
+	if field.Tag.HasOption("join") {
+		internal.Warn.Printf(
+			`%s.%s "join" option must come together with "rel" option`,
+			t.TypeName, field.GoName,
+		)
+	}
+
+	t.FieldMap[field.Name] = field
+	if altName, ok := field.Tag.Option("alt"); ok {
+		t.FieldMap[altName] = field
+	}
+
+	if field.Tag.HasOption("scanonly") {
+		return
+	}
+
+	if _, ok := field.Tag.Options["soft_delete"]; ok {
+		t.SoftDeleteField = field
+		t.UpdateSoftDeleteField = softDeleteFieldUpdater(field)
+	}
+
 	t.Fields = append(t.Fields, field)
 	if field.IsPK {
 		t.PKs = append(t.PKs, field)
 	} else {
 		t.DataFields = append(t.DataFields, field)
 	}
-	t.FieldMap[field.Name] = field
 }
 
-func (t *Table) removeField(field *Field) {
-	t.Fields = removeField(t.Fields, field)
-	if field.IsPK {
-		t.PKs = removeField(t.PKs, field)
-	} else {
-		t.DataFields = removeField(t.DataFields, field)
+func (t *Table) LookupField(name string) *Field {
+	if field, ok := t.FieldMap[name]; ok {
+		return field
 	}
-	delete(t.FieldMap, field.Name)
-}
 
-func (t *Table) fieldWithLock(name string) *Field {
-	t.fieldsMapMu.RLock()
-	field := t.FieldMap[name]
-	t.fieldsMapMu.RUnlock()
-	return field
+	table := t
+	var index []int
+	for {
+		structName, columnName, ok := strings.Cut(name, "__")
+		if !ok {
+			field, ok := table.FieldMap[name]
+			if !ok {
+				return nil
+			}
+			return field.WithIndex(index)
+		}
+		name = columnName
+
+		strct := table.StructMap[structName]
+		if strct == nil {
+			return nil
+		}
+		table = strct.Table
+		index = append(index, strct.Index...)
+	}
 }
 
 func (t *Table) HasField(name string) bool {
@@ -200,59 +441,6 @@ func (t *Table) fieldByGoName(name string) *Field {
 	return nil
 }
 
-func (t *Table) initFields() {
-	t.Fields = make([]*Field, 0, t.Type.NumField())
-	t.FieldMap = make(map[string]*Field, t.Type.NumField())
-	t.addFields(t.Type, "", nil)
-}
-
-func (t *Table) addFields(typ reflect.Type, prefix string, index []int) {
-	for i := 0; i < typ.NumField(); i++ {
-		f := typ.Field(i)
-		unexported := f.PkgPath != ""
-
-		if unexported && !f.Anonymous { // unexported
-			continue
-		}
-		if f.Tag.Get("bun") == "-" {
-			continue
-		}
-
-		if f.Anonymous {
-			if f.Name == "BaseModel" && f.Type == baseModelType {
-				if len(index) == 0 {
-					t.processBaseModelField(f)
-				}
-				continue
-			}
-
-			// If field is an embedded struct, add each field of the embedded struct.
-			fieldType := indirectType(f.Type)
-			if fieldType.Kind() == reflect.Struct {
-				t.addFields(fieldType, "", withIndex(index, f.Index))
-
-				tag := tagparser.Parse(f.Tag.Get("bun"))
-				if tag.HasOption("inherit") || tag.HasOption("extend") {
-					embeddedTable := t.dialect.Tables().Ref(fieldType)
-					t.TypeName = embeddedTable.TypeName
-					t.SQLName = embeddedTable.SQLName
-					t.SQLNameForSelects = embeddedTable.SQLNameForSelects
-					t.Alias = embeddedTable.Alias
-					t.SQLAlias = embeddedTable.SQLAlias
-					t.ModelName = embeddedTable.ModelName
-				}
-				continue
-			}
-		}
-
-		// If field is not a struct, add it.
-		// This will also add any embedded non-struct type as a field.
-		if field := t.newField(f, prefix, index); field != nil {
-			t.addField(field)
-		}
-	}
-}
-
 func (t *Table) processBaseModelField(f reflect.StructField) {
 	tag := tagparser.Parse(f.Tag.Get("bun"))
 
@@ -270,10 +458,18 @@ func (t *Table) processBaseModelField(f reflect.StructField) {
 	}
 
 	if tag.Name != "" {
+		schema, _ := t.schemaFromTagName(tag.Name)
+		t.Schema = schema
+
+		// Eventually, we should only assign the "table" portion as the table name,
+		// which will also require a change in how the table name is appended to queries.
+		// Until that is done, set table name to tag.Name.
 		t.setName(tag.Name)
 	}
 
 	if s, ok := tag.Option("table"); ok {
+		schema, _ := t.schemaFromTagName(s)
+		t.Schema = schema
 		t.setName(s)
 	}
 
@@ -287,59 +483,51 @@ func (t *Table) processBaseModelField(f reflect.StructField) {
 	}
 }
 
-// nolint
-func (t *Table) newField(f reflect.StructField, prefix string, index []int) *Field {
-	tag := tagparser.Parse(f.Tag.Get("bun"))
-
-	if nextPrefix, ok := tag.Option("embed"); ok {
-		fieldType := indirectType(f.Type)
-		if fieldType.Kind() != reflect.Struct {
-			panic(fmt.Errorf("bun: embed %s.%s: got %s, wanted reflect.Struct",
-				t.TypeName, f.Name, fieldType.Kind()))
-		}
-		t.addFields(fieldType, prefix+nextPrefix, withIndex(index, f.Index))
-		return nil
+// schemaFromTagName splits the bun.BaseModel tag name into schema and table name
+// in case it is specified in the "schema"."table" format.
+// Assume default schema if one isn't explicitly specified.
+func (t *Table) schemaFromTagName(name string) (string, string) {
+	schema, table := t.dialect.DefaultSchema(), name
+	if schemaTable := strings.Split(name, "."); len(schemaTable) == 2 {
+		schema, table = schemaTable[0], schemaTable[1]
 	}
+	return schema, table
+}
 
-	sqlName := internal.Underscore(f.Name)
+// nolint
+func (t *Table) newField(sf reflect.StructField, tag tagparser.Tag) *Field {
+	sqlName := internal.Underscore(sf.Name)
 	if tag.Name != "" && tag.Name != sqlName {
 		if isKnownFieldOption(tag.Name) {
 			internal.Warn.Printf(
 				"%s.%s tag name %q is also an option name, is it a mistake? Try column:%s.",
-				t.TypeName, f.Name, tag.Name, tag.Name,
+				t.TypeName, sf.Name, tag.Name, tag.Name,
 			)
 		}
 		sqlName = tag.Name
 	}
+
 	if s, ok := tag.Option("column"); ok {
 		sqlName = s
 	}
-	sqlName = prefix + sqlName
 
 	for name := range tag.Options {
 		if !isKnownFieldOption(name) {
-			internal.Warn.Printf("%s.%s has unknown tag option: %q", t.TypeName, f.Name, name)
+			internal.Warn.Printf("%s.%s has unknown tag option: %q", t.TypeName, sf.Name, name)
 		}
 	}
 
-	index = withIndex(index, f.Index)
-	if field := t.fieldWithLock(sqlName); field != nil {
-		if indexEqual(field.Index, index) {
-			return field
-		}
-		t.removeField(field)
-	}
-
 	field := &Field{
-		StructField: f,
-		IsPtr:       f.Type.Kind() == reflect.Ptr,
+		Table:       t,
+		StructField: sf,
+		IsPtr:       sf.Type.Kind() == reflect.Ptr,
 
 		Tag:          tag,
-		IndirectType: indirectType(f.Type),
-		Index:        index,
+		IndirectType: indirectType(sf.Type),
+		Index:        sf.Index,
 
 		Name:    sqlName,
-		GoName:  f.Name,
+		GoName:  sf.Name,
 		SQLName: t.quoteIdent(sqlName),
 	}
 
@@ -351,6 +539,7 @@ func (t *Table) newField(f reflect.StructField, prefix string, index []int) *Fie
 	}
 	if tag.HasOption("autoincrement") {
 		field.AutoIncrement = true
+		field.NotNull = true
 		field.NullZero = true
 	}
 	if tag.HasOption("identity") {
@@ -358,22 +547,7 @@ func (t *Table) newField(f reflect.StructField, prefix string, index []int) *Fie
 	}
 
 	if v, ok := tag.Options["unique"]; ok {
-		var names []string
-		if len(v) == 1 {
-			// Split the value by comma, this will allow multiple names to be specified.
-			// We can use this to create multiple named unique constraints where a single column
-			// might be included in multiple constraints.
-			names = strings.Split(v[0], ",")
-		} else {
-			names = v
-		}
-
-		for _, uniqueName := range names {
-			if t.Unique == nil {
-				t.Unique = make(map[string][]*Field)
-			}
-			t.Unique[uniqueName] = append(t.Unique[uniqueName], field)
-		}
+		t.addUnique(field, "", v)
 	}
 	if s, ok := tag.Option("default"); ok {
 		field.SQLDefault = s
@@ -386,63 +560,28 @@ func (t *Table) newField(f reflect.StructField, prefix string, index []int) *Fie
 	field.Scan = FieldScanner(t.dialect, field)
 	field.IsZero = zeroChecker(field.StructField.Type)
 
-	if v, ok := tag.Option("alt"); ok {
-		t.FieldMap[v] = field
-	}
-
-	t.allFields = append(t.allFields, field)
-	if tag.HasOption("scanonly") {
-		t.FieldMap[field.Name] = field
-		if field.IndirectType.Kind() == reflect.Struct {
-			t.inlineFields(field, nil)
-		}
-		return nil
-	}
-
-	if _, ok := tag.Options["soft_delete"]; ok {
-		t.SoftDeleteField = field
-		t.UpdateSoftDeleteField = softDeleteFieldUpdater(field)
-	}
-
 	return field
 }
 
 //---------------------------------------------------------------------------------------
 
 func (t *Table) initRelations() {
-	for i := 0; i < len(t.Fields); {
-		f := t.Fields[i]
-		if t.tryRelation(f) {
-			t.Fields = removeField(t.Fields, f)
-			t.DataFields = removeField(t.DataFields, f)
-		} else {
-			i++
-		}
-
-		if f.IndirectType.Kind() == reflect.Struct {
-			t.inlineFields(f, nil)
-		}
+	for _, field := range t.relFields {
+		t.processRelation(field)
 	}
+	t.relFields = nil
 }
 
-func (t *Table) tryRelation(field *Field) bool {
+func (t *Table) processRelation(field *Field) {
 	if rel, ok := field.Tag.Option("rel"); ok {
 		t.initRelation(field, rel)
-		return true
+		return
 	}
 	if field.Tag.HasOption("m2m") {
 		t.addRelation(t.m2mRelation(field))
-		return true
+		return
 	}
-
-	if field.Tag.HasOption("join") {
-		internal.Warn.Printf(
-			`%s.%s "join" option must come together with "rel" option`,
-			t.TypeName, field.GoName,
-		)
-	}
-
-	return false
+	panic("not reached")
 }
 
 func (t *Table) initRelation(field *Field, rel string) {
@@ -470,7 +609,7 @@ func (t *Table) addRelation(rel *Relation) {
 }
 
 func (t *Table) belongsToRelation(field *Field) *Relation {
-	joinTable := t.dialect.Tables().Ref(field.IndirectType)
+	joinTable := t.dialect.Tables().InProgress(field.IndirectType)
 	if err := joinTable.CheckPKs(); err != nil {
 		panic(err)
 	}
@@ -485,7 +624,10 @@ func (t *Table) belongsToRelation(field *Field) *Relation {
 		rel.Condition = field.Tag.Options["join_on"]
 	}
 
-	rel.OnUpdate = "ON UPDATE NO ACTION"
+	if t.dialect.Features().Has(feature.FKDefaultOnAction) {
+		rel.OnUpdate = "ON UPDATE NO ACTION"
+		rel.OnDelete = "ON DELETE NO ACTION"
+	}
 	if onUpdate, ok := field.Tag.Options["on_update"]; ok {
 		if len(onUpdate) > 1 {
 			panic(fmt.Errorf("bun: %s belongs-to %s: on_update option must be a single field", t.TypeName, field.GoName))
@@ -500,7 +642,6 @@ func (t *Table) belongsToRelation(field *Field) *Relation {
 		rel.OnUpdate = s
 	}
 
-	rel.OnDelete = "ON DELETE NO ACTION"
 	if onDelete, ok := field.Tag.Options["on_delete"]; ok {
 		if len(onDelete) > 1 {
 			panic(fmt.Errorf("bun: %s belongs-to %s: on_delete option must be a single field", t.TypeName, field.GoName))
@@ -519,8 +660,8 @@ func (t *Table) belongsToRelation(field *Field) *Relation {
 		for i, baseColumn := range baseColumns {
 			joinColumn := joinColumns[i]
 
-			if f := t.fieldWithLock(baseColumn); f != nil {
-				rel.BaseFields = append(rel.BaseFields, f)
+			if f := t.FieldMap[baseColumn]; f != nil {
+				rel.BasePKs = append(rel.BasePKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s belongs-to %s: %s must have column %s",
@@ -528,8 +669,8 @@ func (t *Table) belongsToRelation(field *Field) *Relation {
 				))
 			}
 
-			if f := joinTable.fieldWithLock(joinColumn); f != nil {
-				rel.JoinFields = append(rel.JoinFields, f)
+			if f := joinTable.FieldMap[joinColumn]; f != nil {
+				rel.JoinPKs = append(rel.JoinPKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s belongs-to %s: %s must have column %s",
@@ -540,17 +681,17 @@ func (t *Table) belongsToRelation(field *Field) *Relation {
 		return rel
 	}
 
-	rel.JoinFields = joinTable.PKs
+	rel.JoinPKs = joinTable.PKs
 	fkPrefix := internal.Underscore(field.GoName) + "_"
 	for _, joinPK := range joinTable.PKs {
 		fkName := fkPrefix + joinPK.Name
-		if fk := t.fieldWithLock(fkName); fk != nil {
-			rel.BaseFields = append(rel.BaseFields, fk)
+		if fk := t.FieldMap[fkName]; fk != nil {
+			rel.BasePKs = append(rel.BasePKs, fk)
 			continue
 		}
 
-		if fk := t.fieldWithLock(joinPK.Name); fk != nil {
-			rel.BaseFields = append(rel.BaseFields, fk)
+		if fk := t.FieldMap[joinPK.Name]; fk != nil {
+			rel.BasePKs = append(rel.BasePKs, fk)
 			continue
 		}
 
@@ -568,7 +709,7 @@ func (t *Table) hasOneRelation(field *Field) *Relation {
 		panic(err)
 	}
 
-	joinTable := t.dialect.Tables().Ref(field.IndirectType)
+	joinTable := t.dialect.Tables().InProgress(field.IndirectType)
 	rel := &Relation{
 		Type:      HasOneRelation,
 		Field:     field,
@@ -582,8 +723,8 @@ func (t *Table) hasOneRelation(field *Field) *Relation {
 	if join, ok := field.Tag.Options["join"]; ok {
 		baseColumns, joinColumns := parseRelationJoin(join)
 		for i, baseColumn := range baseColumns {
-			if f := t.fieldWithLock(baseColumn); f != nil {
-				rel.BaseFields = append(rel.BaseFields, f)
+			if f := t.FieldMap[baseColumn]; f != nil {
+				rel.BasePKs = append(rel.BasePKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s has-one %s: %s must have column %s",
@@ -592,8 +733,8 @@ func (t *Table) hasOneRelation(field *Field) *Relation {
 			}
 
 			joinColumn := joinColumns[i]
-			if f := joinTable.fieldWithLock(joinColumn); f != nil {
-				rel.JoinFields = append(rel.JoinFields, f)
+			if f := joinTable.FieldMap[joinColumn]; f != nil {
+				rel.JoinPKs = append(rel.JoinPKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s has-one %s: %s must have column %s",
@@ -604,17 +745,17 @@ func (t *Table) hasOneRelation(field *Field) *Relation {
 		return rel
 	}
 
-	rel.BaseFields = t.PKs
+	rel.BasePKs = t.PKs
 	fkPrefix := internal.Underscore(t.ModelName) + "_"
 	for _, pk := range t.PKs {
 		fkName := fkPrefix + pk.Name
-		if f := joinTable.fieldWithLock(fkName); f != nil {
-			rel.JoinFields = append(rel.JoinFields, f)
+		if f := joinTable.FieldMap[fkName]; f != nil {
+			rel.JoinPKs = append(rel.JoinPKs, f)
 			continue
 		}
 
-		if f := joinTable.fieldWithLock(pk.Name); f != nil {
-			rel.JoinFields = append(rel.JoinFields, f)
+		if f := joinTable.FieldMap[pk.Name]; f != nil {
+			rel.JoinPKs = append(rel.JoinPKs, f)
 			continue
 		}
 
@@ -638,7 +779,7 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 		))
 	}
 
-	joinTable := t.dialect.Tables().Ref(indirectType(field.IndirectType.Elem()))
+	joinTable := t.dialect.Tables().InProgress(indirectType(field.IndirectType.Elem()))
 	polymorphicValue, isPolymorphic := field.Tag.Option("polymorphic")
 	rel := &Relation{
 		Type:      HasManyRelation,
@@ -662,8 +803,8 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 				continue
 			}
 
-			if f := t.fieldWithLock(baseColumn); f != nil {
-				rel.BaseFields = append(rel.BaseFields, f)
+			if f := t.FieldMap[baseColumn]; f != nil {
+				rel.BasePKs = append(rel.BasePKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s has-many %s: %s must have column %s",
@@ -671,8 +812,8 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 				))
 			}
 
-			if f := joinTable.fieldWithLock(joinColumn); f != nil {
-				rel.JoinFields = append(rel.JoinFields, f)
+			if f := joinTable.FieldMap[joinColumn]; f != nil {
+				rel.JoinPKs = append(rel.JoinPKs, f)
 			} else {
 				panic(fmt.Errorf(
 					"bun: %s has-many %s: %s must have column %s",
@@ -681,7 +822,7 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 			}
 		}
 	} else {
-		rel.BaseFields = t.PKs
+		rel.BasePKs = t.PKs
 		fkPrefix := internal.Underscore(t.ModelName) + "_"
 		if isPolymorphic {
 			polymorphicColumn = fkPrefix + "type"
@@ -689,13 +830,13 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 
 		for _, pk := range t.PKs {
 			joinColumn := fkPrefix + pk.Name
-			if fk := joinTable.fieldWithLock(joinColumn); fk != nil {
-				rel.JoinFields = append(rel.JoinFields, fk)
+			if fk := joinTable.FieldMap[joinColumn]; fk != nil {
+				rel.JoinPKs = append(rel.JoinPKs, fk)
 				continue
 			}
 
-			if fk := joinTable.fieldWithLock(pk.Name); fk != nil {
-				rel.JoinFields = append(rel.JoinFields, fk)
+			if fk := joinTable.FieldMap[pk.Name]; fk != nil {
+				rel.JoinPKs = append(rel.JoinPKs, fk)
 				continue
 			}
 
@@ -708,7 +849,7 @@ func (t *Table) hasManyRelation(field *Field) *Relation {
 	}
 
 	if isPolymorphic {
-		rel.PolymorphicField = joinTable.fieldWithLock(polymorphicColumn)
+		rel.PolymorphicField = joinTable.FieldMap[polymorphicColumn]
 		if rel.PolymorphicField == nil {
 			panic(fmt.Errorf(
 				"bun: %s has-many %s: %s must have polymorphic column %s",
@@ -732,7 +873,7 @@ func (t *Table) m2mRelation(field *Field) *Relation {
 			t.TypeName, field.GoName, field.IndirectType.Kind(),
 		))
 	}
-	joinTable := t.dialect.Tables().Ref(indirectType(field.IndirectType.Elem()))
+	joinTable := t.dialect.Tables().InProgress(indirectType(field.IndirectType.Elem()))
 
 	if err := t.CheckPKs(); err != nil {
 		panic(err)
@@ -760,6 +901,7 @@ func (t *Table) m2mRelation(field *Field) *Relation {
 		JoinTable: joinTable,
 		M2MTable:  m2mTable,
 	}
+	m2mTable.markM2M()
 
 	if field.Tag.HasOption("join_on") {
 		rel.Condition = field.Tag.Options["join_on"]
@@ -795,48 +937,18 @@ func (t *Table) m2mRelation(field *Field) *Relation {
 	}
 
 	leftRel := m2mTable.belongsToRelation(leftField)
-	rel.BaseFields = leftRel.JoinFields
-	rel.M2MBaseFields = leftRel.BaseFields
+	rel.BasePKs = leftRel.JoinPKs
+	rel.M2MBasePKs = leftRel.BasePKs
 
 	rightRel := m2mTable.belongsToRelation(rightField)
-	rel.JoinFields = rightRel.JoinFields
-	rel.M2MJoinFields = rightRel.BaseFields
+	rel.JoinPKs = rightRel.JoinPKs
+	rel.M2MJoinPKs = rightRel.BasePKs
 
 	return rel
 }
 
-func (t *Table) inlineFields(field *Field, seen map[reflect.Type]struct{}) {
-	if seen == nil {
-		seen = map[reflect.Type]struct{}{t.Type: {}}
-	}
-
-	if _, ok := seen[field.IndirectType]; ok {
-		return
-	}
-	seen[field.IndirectType] = struct{}{}
-
-	joinTable := t.dialect.Tables().Ref(field.IndirectType)
-	for _, f := range joinTable.allFields {
-		f = f.Clone()
-		f.GoName = field.GoName + "_" + f.GoName
-		f.Name = field.Name + "__" + f.Name
-		f.SQLName = t.quoteIdent(f.Name)
-		f.Index = withIndex(field.Index, f.Index)
-
-		t.fieldsMapMu.Lock()
-		if _, ok := t.FieldMap[f.Name]; !ok {
-			t.FieldMap[f.Name] = f
-		}
-		t.fieldsMapMu.Unlock()
-
-		if f.IndirectType.Kind() != reflect.Struct {
-			continue
-		}
-
-		if _, ok := seen[f.IndirectType]; !ok {
-			t.inlineFields(f, seen)
-		}
-	}
+func (t *Table) markM2M() {
+	t.IsM2MTable = true
 }
 
 //------------------------------------------------------------------------------
@@ -890,11 +1002,12 @@ func isKnownTableOption(name string) bool {
 func isKnownFieldOption(name string) bool {
 	switch name {
 	case "column",
-		"alias",
+		"alt",
 		"type",
 		"array",
 		"hstore",
 		"composite",
+		"multirange",
 		"json_use_number",
 		"msgpack",
 		"notnull",
@@ -931,15 +1044,6 @@ func isKnownFKRule(name string) bool {
 	return false
 }
 
-func removeField(fields []*Field, field *Field) []*Field {
-	for i, f := range fields {
-		if f == field {
-			return append(fields[:i], fields[i+1:]...)
-		}
-	}
-	return fields
-}
-
 func parseRelationJoin(join []string) ([]string, []string) {
 	var ss []string
 	if len(join) == 1 {
@@ -1026,7 +1130,7 @@ func softDeleteFieldUpdaterFallback(field *Field) func(fv reflect.Value, tm time
 	}
 }
 
-func withIndex(a, b []int) []int {
+func makeIndex(a, b []int) []int {
 	dest := make([]int, 0, len(a)+len(b))
 	dest = append(dest, a...)
 	dest = append(dest, b...)
